use sprinklers_core::model::{ProgramId, SectionId};

#[derive(Clone, Debug)]
pub struct Topics<T>
where
    T: AsRef<str>,
{
    prefix: T,
}

impl<T> Topics<T>
where
    T: AsRef<str>,
{
    pub fn new(prefix: T) -> Self {
        Self { prefix }
    }

    pub fn connected(&self) -> String {
        format!("{}/connected", self.prefix.as_ref())
    }

    pub fn sections(&self) -> String {
        format!("{}/sections", self.prefix.as_ref())
    }

    pub fn section_data(&self, section_id: SectionId) -> String {
        format!("{}/sections/{}", self.prefix.as_ref(), section_id)
    }

    pub fn section_state(&self, section_id: SectionId) -> String {
        format!("{}/sections/{}/state", self.prefix.as_ref(), section_id)
    }

    pub fn programs(&self) -> String {
        format!("{}/programs", self.prefix.as_ref())
    }

    pub fn program_data(&self, program_id: ProgramId) -> String {
        format!("{}/programs/{}", self.prefix.as_ref(), program_id)
    }

    pub fn program_running(&self, program_id: ProgramId) -> String {
        format!("{}/programs/{}/running", self.prefix.as_ref(), program_id)
    }

    pub fn program_next_run(&self, program_id: ProgramId) -> String {
        // TODO: reconcile naming convention
        format!("{}/programs/{}/nextRun", self.prefix.as_ref(), program_id)
    }

    pub fn section_runner(&self) -> String {
        format!("{}/section_runner", self.prefix.as_ref())
    }

    pub fn requests(&self) -> String {
        format!("{}/requests", self.prefix.as_ref())
    }

    pub fn responses(&self) -> String {
        format!("{}/responses", self.prefix.as_ref())
    }
}
